/* 
 * new.H - Memory management functions.
 * 
 * Author:	Robert Mecklenburg
 * 		Computer Science Dept.
 * 		University of Utah
 * Date:	Wed Jun 29, 1994
 *
 * Copyright (c) 1994 The University of Utah and
 * the Center for Software Science (CSS).  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the Center
 * for Software Science at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSS ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSS DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSS requests users of this software to return to css-dist@cs.utah.edu any
 * improvements that they make and grant CSS redistribution rights.
 */

#ifndef NEW_H_
#define NEW_H_

/*****************************************************************
 * mem_elem_c
 *
 * Describe an element in a linked list of memory chunks.  In fact,
 * this structure is allocated as a prefix to every malloc performed.
 *
 * count_	A unique identifier for each memory chunk.
 * size_	The number of bytes in the chunk not including the
 * 		mem_elem_c header or the trailing cookie.  Actually
 * 		consists of two bit fields: a single bit indicating if
 * 		the memory chunk has been freed and a 31 bit size field.
 * file_	The file in which the allocation occurred.
 * line_	The line number at which the allocation occurred.
 * next_	The next chunk.
 * prev_	The previous chunk.
 */
class mem_elem_c
{
    friend class	mem_list_c;
    friend void		mem_list_report();
    friend void		mem_dbg( mem_elem_c * m );

    enum
    {
	FREE_MASK  = 1 << 31,			/* Been freed bit. */
	SIZE_MASK  = ~FREE_MASK,
    };

    int			count() const		{ return count_; }
    int			size() const		{ return size_ & SIZE_MASK; }
    int			freed() const		{ return size_ & FREE_MASK; }
    void		set_free( int f );
    void		set_control( int cnt,
				     int sz,
				     int fr,
				     const char * fi,
				     int li );
    void		clear( unsigned char val );
    int			valid() const;
    void		set_cookie();
    int			check_cookie() const;
    void		report( int omit_hdr, int chk ) const;

    unsigned int	count_;
    unsigned int	size_;
    const char *	file_;
    int			line_;
    mem_elem_c *	next_;
    mem_elem_c *	prev_;
};


/*****************************************************************
 * mem_list_c
 *
 * A class of all currently allocated memory.  The body of the class
 * is a doubly linked list of memory elements.  The class also
 * contains control information as static data members.
 *
 * count	The next memory identifier to use.
 * max_size	The largest memory chunk allocated so far.
 * watch_new	Sing out when we allocate this unit.
 * watch_del	Sing out when we deallocate this unit.
 * head_	The first member of the memory element list.
 */
class mem_list_c
{
public:
    mem_list_c();
    ~mem_list_c();

    mem_elem_c *	head() const	{ return head_; }

    void		initialize();
    void		insert( mem_elem_c * s,
			        size_t sz,
			        const char * f,
			        int l );
    void		remove( mem_elem_c * );
    friend void		mem_list_report();

    static int 		count;
    static int		max_size;
    static int		watch_new;
    static int		watch_del;

private:
    mem_elem_c *	head_;
};

#endif /* NEW_H_ */
